@@ -1,10 +1,12 @@
-//@line 39 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
+//@line 42 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
 
 const Ci = Components.interfaces;
 const Cc = Components.classes;
 const Cr = Components.results;
 const Cu = Components.utils;
 
+const XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
+
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource:///modules/distribution.js");
 
@@ -32,25 +34,46 @@ const BrowserGlueServiceFactory = {
 // Constructor
 
 function BrowserGlue() {
+
+  this.__defineGetter__("_prefs", function() {
+    delete this._prefs;
+    return this._prefs = Cc["@mozilla.org/preferences-service;1"].
+                         getService(Ci.nsIPrefBranch);
+  });
+
+  this.__defineGetter__("_bundleService", function() {
+    delete this._bundleService;
+    return this._bundleService = Cc["@mozilla.org/intl/stringbundle;1"].
+                                 getService(Ci.nsIStringBundleService);
+  });
+
+  this.__defineGetter__("_idleService", function() {
+    delete this._idleService;
+    return this._idleService = Cc["@mozilla.org/widget/idleservice;1"].
+                           getService(Ci.nsIIdleService);
+  });
+
+  this.__defineGetter__("_observerService", function() {
+    delete this._observerService;
+    return this._observerService = Cc['@mozilla.org/observer-service;1'].
+                                   getService(Ci.nsIObserverService);
+  });
+
   this._init();
 }
 
 BrowserGlue.prototype = {
-  __prefs: null,
-  get _prefs() {
-    if (!this.__prefs)
-      this.__prefs = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
-    return this.__prefs;
-  },
-
+  
   _saveSession: false,
 
   _setPrefToSaveSession: function()
   {
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
-    prefBranch.setBoolPref("browser.sessionstore.resume_session_once", true);
+    this._prefs.setBoolPref("browser.sessionstore.resume_session_once", true);
+
+    // This method can be called via [NSApplication terminate:] on Mac, which
+    // ends up causing prefs not to be flushed to disk, so we need to do that
+    // explicitly here. See bug 497652.
+    this._prefs.QueryInterface(Ci.nsIPrefService).savePrefFile(null);
   },
 
   // nsIObserver implementation 
@@ -60,9 +83,6 @@ BrowserGlue.prototype = {
       case "xpcom-shutdown":
         this._dispose();
         break;
-      case "quit-application": 
-        this._onProfileShutdown();
-        break;
       case "prefservice:after-app-defaults":
         this._onAppDefaults();
         break;
@@ -86,14 +106,30 @@ BrowserGlue.prototype = {
         if (this._saveSession) {
           this._setPrefToSaveSession();
         }
+        // Everything that uses Places during shutdown should be here, since
+        // on quit-application Places database connection will be closed
+        // and history synchronization could fail.
+        this._onProfileShutdown();
         break;
       case "session-save":
         this._setPrefToSaveSession();
         subject.QueryInterface(Ci.nsISupportsPRBool);
         subject.data = true;
         break;
+      case "places-init-complete":
+        this._initPlaces();
+        this._observerService.removeObserver(this, "places-init-complete");
+        // no longer needed, since history was initialized completely.
+        this._observerService.removeObserver(this, "places-database-locked");
+        break;
+      case "places-database-locked":
+        this._isPlacesDatabaseLocked = true;
+        // stop observing, so further attempts to load history service
+        // do not show the prompt.
+        this._observerService.removeObserver(this, "places-database-locked");
+        break;
       case "idle":
-        if (this.idleService.idleTime > BOOKMARKS_ARCHIVE_IDLE_TIME * 1000) {
+        if (this._idleService.idleTime > BOOKMARKS_ARCHIVE_IDLE_TIME * 1000) {
           // Back up bookmarks.
           this._archiveBookmarks();
         }
@@ -105,9 +141,7 @@ BrowserGlue.prototype = {
   _init: function() 
   {
     // observer registration
-    const osvr = Cc['@mozilla.org/observer-service;1'].
-                 getService(Ci.nsIObserverService);
-    osvr.addObserver(this, "quit-application", false);
+    const osvr = this._observerService;
     osvr.addObserver(this, "xpcom-shutdown", false);
     osvr.addObserver(this, "prefservice:after-app-defaults", false);
     osvr.addObserver(this, "final-ui-startup", false);
@@ -116,15 +150,15 @@ BrowserGlue.prototype = {
     osvr.addObserver(this, "quit-application-requested", false);
     osvr.addObserver(this, "quit-application-granted", false);
     osvr.addObserver(this, "session-save", false);
+    osvr.addObserver(this, "places-init-complete", false);
+    osvr.addObserver(this, "places-database-locked", false);
   },
 
   // cleanup (called on application shutdown)
   _dispose: function() 
   {
     // observer removal 
-    const osvr = Cc['@mozilla.org/observer-service;1'].
-                 getService(Ci.nsIObserverService);
-    osvr.removeObserver(this, "quit-application");
+    const osvr = this._observerService;
     osvr.removeObserver(this, "xpcom-shutdown");
     osvr.removeObserver(this, "prefservice:after-app-defaults");
     osvr.removeObserver(this, "final-ui-startup");
@@ -157,9 +191,6 @@ BrowserGlue.prototype = {
                     "_blank", "chrome,centerscreen,modal,resizable=no", null);
     }
 
-    // initialize Places
-    this._initPlaces();
-
     // apply distribution customizations
     // prefs are applied in _onAppDefaults()
     var distro = new DistributionCustomizer();
@@ -168,16 +199,14 @@ BrowserGlue.prototype = {
     // handle any UI migration
     this._migrateUI();
 
-    const osvr = Cc['@mozilla.org/observer-service;1'].
-                 getService(Ci.nsIObserverService);
-    osvr.notifyObservers(null, "browser-ui-startup-complete", "");
+    this._observerService.notifyObservers(null, "browser-ui-startup-complete", "");
   },
 
   // profile shutdown handler (contains profile cleanup routines)
   _onProfileShutdown: function() 
   {
     this._shutdownPlaces();
-    this.idleService.removeIdleObserver(this, BOOKMARKS_ARCHIVE_IDLE_TIME);
+    this._idleService.removeIdleObserver(this, BOOKMARKS_ARCHIVE_IDLE_TIME);
     this.Sanitizer.onShutdown();
   },
 
@@ -188,10 +217,8 @@ BrowserGlue.prototype = {
     if (this._shouldShowRights())
       this._showRightsNotification();
 
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
     // If new add-ons were installed during startup open the add-ons manager.
-    if (prefBranch.prefHasUserValue(PREF_EM_NEW_ADDONS_LIST)) {
+    if (this._prefs.prefHasUserValue(PREF_EM_NEW_ADDONS_LIST)) {
       var args = Cc["@mozilla.org/supports-array;1"].
                  createInstance(Ci.nsISupportsArray);
       var str = Cc["@mozilla.org/supports-string;1"].
@@ -200,14 +227,23 @@ BrowserGlue.prototype = {
       args.AppendElement(str);
       var str = Cc["@mozilla.org/supports-string;1"].
                 createInstance(Ci.nsISupportsString);
-      str.data = prefBranch.getCharPref(PREF_EM_NEW_ADDONS_LIST);
+      str.data = this._prefs.getCharPref(PREF_EM_NEW_ADDONS_LIST);
       args.AppendElement(str);
       const EMURL = "chrome://mozapps/content/extensions/extensions.xul";
       const EMFEATURES = "chrome,menubar,extra-chrome,toolbar,dialog=no,resizable";
       var ww = Cc["@mozilla.org/embedcomp/window-watcher;1"].
                getService(Ci.nsIWindowWatcher);
       ww.openWindow(null, EMURL, "_blank", EMFEATURES, args);
-      prefBranch.clearUserPref(PREF_EM_NEW_ADDONS_LIST);
+      this._prefs.clearUserPref(PREF_EM_NEW_ADDONS_LIST);
+    }
+
+    // Load the "more info" page for a locked places.sqlite
+    // This property is set earlier in the startup process:
+    // nsPlacesDBFlush loads after profile-after-change and initializes
+    // the history service, which sends out places-database-locked
+    // which sets this property.
+    if (this._isPlacesDatabaseLocked) {
+      this._showPlacesLockedNotificationBox();
     }
   },
 
@@ -239,36 +275,36 @@ BrowserGlue.prototype = {
     if (aQuitType != "restart")
       aQuitType = "quit";
 
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
     var showPrompt = true;
     try {
       // browser.warnOnQuit is a hidden global boolean to override all quit prompts
       // browser.warnOnRestart specifically covers app-initiated restarts where we restart the app
       // browser.tabs.warnOnClose is the global "warn when closing multiple tabs" pref
 
-      var sessionWillBeSaved = prefBranch.getIntPref("browser.startup.page") == 3 ||
-                               prefBranch.getBoolPref("browser.sessionstore.resume_session_once");
-      if (sessionWillBeSaved || !prefBranch.getBoolPref("browser.warnOnQuit"))
+      var sessionWillBeSaved = this._prefs.getIntPref("browser.startup.page") == 3 ||
+                               this._prefs.getBoolPref("browser.sessionstore.resume_session_once");
+      if (sessionWillBeSaved || !this._prefs.getBoolPref("browser.warnOnQuit"))
         showPrompt = false;
       else if (aQuitType == "restart")
-        showPrompt = prefBranch.getBoolPref("browser.warnOnRestart");
+        showPrompt = this._prefs.getBoolPref("browser.warnOnRestart");
       else
-        showPrompt = prefBranch.getBoolPref("browser.tabs.warnOnClose");
+        showPrompt = this._prefs.getBoolPref("browser.tabs.warnOnClose");
     } catch (ex) {}
 
-    if (!showPrompt)
+    // Never show a prompt inside the private browsing mode
+    var inPrivateBrowsing = Cc["@mozilla.org/privatebrowsing;1"].
+                            getService(Ci.nsIPrivateBrowsingService).
+                            privateBrowsingEnabled;
+    if (!showPrompt || inPrivateBrowsing)
       return false;
 
     var buttonChoice = 0;
-    var bundleService = Cc["@mozilla.org/intl/stringbundle;1"].
-                        getService(Ci.nsIStringBundleService);
-    var quitBundle = bundleService.createBundle("chrome://browser/locale/quitDialog.properties");
-    var brandBundle = bundleService.createBundle("chrome://branding/locale/brand.properties");
+    var quitBundle = this._bundleService.createBundle("chrome://browser/locale/quitDialog.properties");
+    var brandBundle = this._bundleService.createBundle("chrome://branding/locale/brand.properties");
 
     var appName = brandBundle.GetStringFromName("brandShortName");
     var quitDialogTitle = quitBundle.formatStringFromName(aQuitType + "DialogTitle",
-                                                            [appName], 1);
+                                                          [appName], 1);
 
     var message;
     if (aQuitType == "restart")
@@ -308,7 +344,7 @@ BrowserGlue.prototype = {
     switch (buttonChoice) {
     case 2: // Quit
       if (neverAsk.value)
-        prefBranch.setBoolPref("browser.tabs.warnOnClose", false);
+        this._prefs.setBoolPref("browser.tabs.warnOnClose", false);
       break;
     case 1: // Cancel
       aCancelQuit.QueryInterface(Ci.nsISupportsPRBool);
@@ -318,10 +354,10 @@ BrowserGlue.prototype = {
       this._saveSession = true;
       if (neverAsk.value) {
         if (aQuitType == "restart")
-          prefBranch.setBoolPref("browser.warnOnRestart", false);
+          this._prefs.setBoolPref("browser.warnOnRestart", false);
         else {
           // always save state when shutting down
-          prefBranch.setIntPref("browser.startup.page", 3);
+          this._prefs.setIntPref("browser.startup.page", 3);
         }
       }
       break;
@@ -347,7 +383,7 @@ BrowserGlue.prototype = {
       return !this._prefs.getBoolPref("browser.EULA.override");
     } catch (e) { }
 
-//@line 391 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
+//@line 430 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
 
     // Look to see if the user has seen the current version or not.
     var currentVersion = this._prefs.getIntPref("browser.rights.version");
@@ -367,19 +403,17 @@ BrowserGlue.prototype = {
 
   _showRightsNotification : function () {
     // Stick the notification onto the selected tab of the active browser window.
-    var win = this._getMostRecentBrowserWindow();
+    var win = this.getMostRecentBrowserWindow();
     var browser = win.gBrowser; // for closure in notification bar callback
     var notifyBox = browser.getNotificationBox();
 
-    var bundleService = Cc["@mozilla.org/intl/stringbundle;1"].
-                        getService(Ci.nsIStringBundleService);
-    var brandBundle  = bundleService.createBundle("chrome://branding/locale/brand.properties");
-    var rightsBundle = bundleService.createBundle("chrome://browser/locale/aboutRights.properties");
+    var brandBundle  = this._bundleService.createBundle("chrome://branding/locale/brand.properties");
+    var rightsBundle = this._bundleService.createBundle("chrome://browser/locale/aboutRights.properties");
 
-    var buttonLabel     = rightsBundle.GetStringFromName("buttonLabel");
-    var buttonAccessKey = rightsBundle.GetStringFromName("buttonAccessKey");
-    var productName     = brandBundle.GetStringFromName("brandFullName");
-    var notifyText      = rightsBundle.formatStringFromName("notifyText", [productName], 1);
+    var buttonLabel      = rightsBundle.GetStringFromName("buttonLabel");
+    var buttonAccessKey  = rightsBundle.GetStringFromName("buttonAccessKey");
+    var productName      = brandBundle.GetStringFromName("brandFullName");
+    var notifyRightsText = rightsBundle.formatStringFromName("notifyRightsText", [productName], 1);
     
     var buttons = [
                     {
@@ -396,8 +430,8 @@ BrowserGlue.prototype = {
     var currentVersion = this._prefs.getIntPref("browser.rights.version");
     this._prefs.setBoolPref("browser.rights." + currentVersion + ".shown", true);
 
-    var box = notifyBox.appendNotification(notifyText, "about-rights", null, notifyBox.PRIORITY_INFO_LOW, buttons);
-    box.persistence = 3; // // arbitrary number, just so bar sticks around for a bit
+    var box = notifyBox.appendNotification(notifyRightsText, "about-rights", null, notifyBox.PRIORITY_INFO_LOW, buttons);
+    box.persistence = 3; // arbitrary number, just so bar sticks around for a bit
   },
 
   // returns the (cached) Sanitizer constructor
@@ -411,59 +445,99 @@ BrowserGlue.prototype = {
     return Sanitizer;
   },
 
-  _idleService: null,
-  get idleService() {
-    if (!this._idleService)
-      this._idleService = Cc["@mozilla.org/widget/idleservice;1"].
-                          getService(Ci.nsIIdleService);
-    return this._idleService;
-  },
-
   /**
    * Initialize Places
-   * - imports the bookmarks html file if bookmarks datastore is empty
+   * - imports the bookmarks html file if bookmarks database is empty, try to
+   *   restore bookmarks from a JSON backup if the backend indicates that the
+   *   database was corrupt.
+   *
+   * These prefs can be set up by the frontend:
+   *
+   * WARNING: setting these preferences to true will overwite existing bookmarks
    *
-   * These prefs are set by the backend services upon creation (or recreation)
-   * of the Places db:
    * - browser.places.importBookmarksHTML
-   *   Set to false by the history service to indicate we need to re-import.
+   *   Set to true will import the bookmarks.html file from the profile folder.
    * - browser.places.smartBookmarksVersion
    *   Set during HTML import to indicate that Smart Bookmarks were created.
    *   Set to -1 to disable Smart Bookmarks creation.
    *   Set to 0 to restore current Smart Bookmarks.
-   *
-   * These prefs are set up by the frontend:
    * - browser.bookmarks.restore_default_bookmarks
    *   Set to true by safe-mode dialog to indicate we must restore default
    *   bookmarks.
    */
   _initPlaces: function bg__initPlaces() {
-    // we need to instantiate the history service before checking
-    // the browser.places.importBookmarksHTML pref, as
-    // nsNavHistory::ForceMigrateBookmarksDB() will set that pref
-    // if we need to force a migration (due to a schema change)
+    // We must instantiate the history service since it will tell us if we
+    // need to import or restore bookmarks due to first-run, corruption or
+    // forced migration (due to a major schema change).
     var histsvc = Cc["@mozilla.org/browser/nav-history-service;1"].
                   getService(Ci.nsINavHistoryService);
 
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
+    // If the database is corrupt or has been newly created we should
+    // import bookmarks.
+    var databaseStatus = histsvc.databaseStatus;
+    var importBookmarks = databaseStatus == histsvc.DATABASE_STATUS_CREATE ||
+                          databaseStatus == histsvc.DATABASE_STATUS_CORRUPT;
 
-    var importBookmarks = false;
-    var restoreDefaultBookmarks = false;
+    if (databaseStatus == histsvc.DATABASE_STATUS_CREATE) {
+      // If the database has just been created, but we already have any
+      // bookmark, this is not the initial import.  This can happen after a
+      // migration from a different browser since migrators run before us.
+      // In such a case we should not import, unless some pref has been set.
+      var bmsvc = Cc["@mozilla.org/browser/nav-bookmarks-service;1"].
+                  getService(Ci.nsINavBookmarksService);
+      if (bmsvc.getIdForItemAt(bmsvc.bookmarksMenuFolder, 0) != -1 ||
+          bmsvc.getIdForItemAt(bmsvc.toolbarFolder, 0) != -1)
+        importBookmarks = false;
+    }
+
+    // Check if user or an extension has required to import bookmarks.html
+    var importBookmarksHTML = false;
     try {
-      restoreDefaultBookmarks = prefBranch.getBoolPref("browser.bookmarks.restore_default_bookmarks");
+      importBookmarksHTML =
+        this._prefs.getBoolPref("browser.places.importBookmarksHTML");
+      if (importBookmarksHTML)
+        importBookmarks = true;
     } catch(ex) {}
 
-    if (restoreDefaultBookmarks) {
-      // Ensure that we already have a bookmarks backup for today
-      this._archiveBookmarks();
-      // we will restore bookmarks from html
-      importBookmarks = true;
-    }
-    else {
-      try {
-        importBookmarks = prefBranch.getBoolPref("browser.places.importBookmarksHTML");
-      } catch(ex) {}
+    // Check if Safe Mode or the user has required to restore bookmarks from
+    // default profile's bookmarks.html
+    var restoreDefaultBookmarks = false;
+    try {
+      restoreDefaultBookmarks =
+        this._prefs.getBoolPref("browser.bookmarks.restore_default_bookmarks");
+      if (restoreDefaultBookmarks) {
+        // Ensure that we already have a bookmarks backup for today
+        this._archiveBookmarks();
+        importBookmarks = true;
+      }
+    } catch(ex) {}
+
+    // If the user did not require to restore default bookmarks, or import
+    // from bookmarks.html, we will try to restore from JSON
+    if (importBookmarks && !restoreDefaultBookmarks && !importBookmarksHTML) {
+      // get latest JSON backup
+      Cu.import("resource://gre/modules/utils.js");
+      var bookmarksBackupFile = PlacesUtils.getMostRecentBackup();
+      if (bookmarksBackupFile && bookmarksBackupFile.leafName.match("\.json$")) {
+        // restore from JSON backup
+        PlacesUtils.restoreBookmarksFromJSONFile(bookmarksBackupFile);
+        importBookmarks = false;
+      }
+      else {
+        // We have created a new database but we don't have any backup available
+        importBookmarks = true;
+        var dirService = Cc["@mozilla.org/file/directory_service;1"].
+                         getService(Ci.nsIProperties);
+        var bookmarksHTMLFile = dirService.get("BMarks", Ci.nsILocalFile);
+        if (bookmarksHTMLFile.exists()) {
+          // If bookmarks.html is available in current profile import it...
+          importBookmarksHTML = true;
+        }
+        else {
+          // ...otherwise we will restore defaults
+          restoreDefaultBookmarks = true;
+        }
+      }
     }
 
     if (!importBookmarks) {
@@ -472,31 +546,34 @@ BrowserGlue.prototype = {
       this.ensurePlacesDefaultQueriesInitialized();
     }
     else {
-      // get latest backup
-      Cu.import("resource://gre/modules/utils.js");
-      var bookmarksFile = PlacesUtils.getMostRecentBackup();
+      // ensurePlacesDefaultQueriesInitialized() is called by import.
+      // Don't try to recreate smart bookmarks if autoExportHTML is true or
+      // smart bookmarks are disabled.
+      var autoExportHTML = false;
+      try {
+        autoExportHTML = this._prefs.getBoolPref("browser.bookmarks.autoExportHTML");
+      } catch(ex) {}
+      var smartBookmarksVersion = 0;
+      try {
+        smartBookmarksVersion = this._prefs.getIntPref("browser.places.smartBookmarksVersion");
+      } catch(ex) {}
+      if (!autoExportHTML && smartBookmarksVersion != -1)
+        this._prefs.setIntPref("browser.places.smartBookmarksVersion", 0);
 
-      if (!restoreDefaultBookmarks &&
-          bookmarksFile && bookmarksFile.leafName.match("\.json$")) {
-        // restore a JSON backup
-        PlacesUtils.restoreBookmarksFromJSONFile(bookmarksFile);
+      // Get bookmarks.html file location
+      var dirService = Cc["@mozilla.org/file/directory_service;1"].
+                       getService(Ci.nsIProperties);
+
+      var bookmarksFile = null;
+      if (restoreDefaultBookmarks) {
+        // User wants to restore bookmarks.html file from default profile folder
+        bookmarksFile = dirService.get("profDef", Ci.nsILocalFile);
+        bookmarksFile.append("bookmarks.html");
       }
-      else {
-        // if there's no JSON backup or we are restoring default bookmarks
-
-        // ensurePlacesDefaultQueriesInitialized() is called by import.
-        prefBranch.setIntPref("browser.places.smartBookmarksVersion", 0);
-
-        var dirService = Cc["@mozilla.org/file/directory_service;1"].
-                         getService(Ci.nsIProperties);
-
-        var bookmarksFile = dirService.get("BMarks", Ci.nsILocalFile);
-        if (restoreDefaultBookmarks || !bookmarksFile.exists()) {
-          // get bookmarks.html file from default profile folder
-          bookmarksFile = dirService.get("profDef", Ci.nsILocalFile);
-          bookmarksFile.append("bookmarks.html");
-        }
+      else
+        bookmarksFile = dirService.get("BMarks", Ci.nsILocalFile);
 
+      if (bookmarksFile.exists()) {
         // import the file
         try {
           var importer = Cc["@mozilla.org/browser/places/import-export-service;1"].
@@ -504,18 +581,23 @@ BrowserGlue.prototype = {
           importer.importHTMLFromFile(bookmarksFile, true /* overwrite existing */);
         } catch (err) {
           // Report the error, but ignore it.
-          Cu.reportError(err);
+          Cu.reportError("Bookmarks.html file could be corrupt. " + err);
         }
-        prefBranch.setBoolPref("browser.places.importBookmarksHTML", false);
-        if (restoreDefaultBookmarks)
-          prefBranch.setBoolPref("browser.bookmarks.restore_default_bookmarks",
-                                 false);
       }
+      else
+        Cu.reportError("Unable to find bookmarks.html file.");
+
+      // Reset preferences, so we won't try to import again at next run
+      if (importBookmarksHTML)
+        this._prefs.setBoolPref("browser.places.importBookmarksHTML", false);
+      if (restoreDefaultBookmarks)
+        this._prefs.setBoolPref("browser.bookmarks.restore_default_bookmarks",
+                                false);
     }
 
     // Initialize bookmark archiving on idle.
     // Once a day, either on idle or shutdown, bookmarks are backed up.
-    this.idleService.addIdleObserver(this, BOOKMARKS_ARCHIVE_IDLE_TIME);
+    this._idleService.addIdleObserver(this, BOOKMARKS_ARCHIVE_IDLE_TIME);
   },
 
   /**
@@ -532,11 +614,9 @@ BrowserGlue.prototype = {
 
     // Backup bookmarks to bookmarks.html to support apps that depend
     // on the legacy format.
-    var prefs = Cc["@mozilla.org/preferences-service;1"].
-                getService(Ci.nsIPrefBranch);
     var autoExportHTML = false;
     try {
-      autoExportHTML = prefs.getBoolPref("browser.bookmarks.autoExportHTML");
+      autoExportHTML = this._prefs.getBoolPref("browser.bookmarks.autoExportHTML");
     } catch(ex) {
       Components.utils.reportError(ex);
     }
@@ -561,22 +641,56 @@ BrowserGlue.prototype = {
     if (!lastBackup ||
         Date.now() - lastBackup.lastModifiedTime > BOOKMARKS_ARCHIVE_INTERVAL) {
       var maxBackups = 5;
-      var prefs = Cc["@mozilla.org/preferences-service;1"].
-                  getService(Ci.nsIPrefBranch);
       try {
-        maxBackups = prefs.getIntPref("browser.bookmarks.max_backups");
+        maxBackups = this._prefs.getIntPref("browser.bookmarks.max_backups");
       } catch(ex) {}
 
       PlacesUtils.archiveBookmarksFile(maxBackups, false /* don't force */);
     }
   },
 
-  _migrateUI: function bg__migrateUI() {
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
+  /**
+   * Show the notificationBox for a locked places database.
+   */
+  _showPlacesLockedNotificationBox: function nsBrowserGlue__showPlacesLockedNotificationBox() {
+    var brandBundle  = this._bundleService.createBundle("chrome://branding/locale/brand.properties");
+    var applicationName = brandBundle.GetStringFromName("brandShortName");
+    var placesBundle = this._bundleService.createBundle("chrome://browser/locale/places/places.properties");
+    var title = placesBundle.GetStringFromName("lockPrompt.title");
+    var text = placesBundle.formatStringFromName("lockPrompt.text", [applicationName], 1);
+    var buttonText = placesBundle.GetStringFromName("lockPromptInfoButton.label");
+    var accessKey = placesBundle.GetStringFromName("lockPromptInfoButton.accessKey");
 
+    var helpTopic = "places-locked";
+    var url = Cc["@mozilla.org/toolkit/URLFormatterService;1"].
+              getService(Components.interfaces.nsIURLFormatter).
+              formatURLPref("app.support.baseURL");
+    url += helpTopic;
+
+    var browser = this.getMostRecentBrowserWindow().gBrowser;
+
+    var buttons = [
+                    {
+                      label:     buttonText,
+                      accessKey: accessKey,
+                      popup:     null,
+                      callback:  function(aNotificationBar, aButton) {
+                        browser.selectedTab = browser.addTab(url);
+                      }
+                    }
+                  ];
+
+    var notifyBox = browser.getNotificationBox();
+    var box = notifyBox.appendNotification(text, title, null,
+                                           notifyBox.PRIORITY_CRITICAL_MEDIUM,
+                                           buttons);
+    box.persistence = -1; // Until user closes it
+  },
+
+  _migrateUI: function bg__migrateUI() {
     var migration = 0;
     try {
-      migration = prefBranch.getIntPref("browser.migration.version");
+      migration = this._prefs.getIntPref("browser.migration.version");
     } catch(ex) {}
 
     if (migration == 0) {
@@ -620,7 +734,7 @@ BrowserGlue.prototype = {
       this._dataSource = null;
 
       // update the migration version
-      prefBranch.setIntPref("browser.migration.version", 1);
+      this._prefs.setIntPref("browser.migration.version", 1);
     }
   },
 
@@ -665,14 +779,11 @@ BrowserGlue.prototype = {
     // XXX should this be a pref?  see bug #399268
     const MAX_RESULTS = 10;
 
-    var prefBranch = Cc["@mozilla.org/preferences-service;1"].
-                     getService(Ci.nsIPrefBranch);
-
     // get current smart bookmarks version
     // By default, if the pref is not set up, we must create Smart Bookmarks
     var smartBookmarksCurrentVersion = 0;
     try {
-      smartBookmarksCurrentVersion = prefBranch.getIntPref(SMART_BOOKMARKS_PREF);
+      smartBookmarksCurrentVersion = this._prefs.getIntPref(SMART_BOOKMARKS_PREF);
     } catch(ex) {}
 
     // bail out if we don't have to create or update Smart Bookmarks
@@ -686,10 +797,6 @@ BrowserGlue.prototype = {
                   getService(Ci.nsIAnnotationService);
 
     var callback = {
-      _placesBundle: Cc["@mozilla.org/intl/stringbundle;1"].
-                     getService(Ci.nsIStringBundleService).
-                     createBundle("chrome://browser/locale/places/places.properties"),
-
       _uri: function(aSpec) {
         return Cc["@mozilla.org/network/io-service;1"].
                getService(Ci.nsIIOService).
@@ -701,10 +808,14 @@ BrowserGlue.prototype = {
         var bookmarksMenuIndex = 0;
         var bookmarksToolbarIndex = 0;
 
+        var placesBundle = Cc["@mozilla.org/intl/stringbundle;1"].
+                           getService(Ci.nsIStringBundleService).
+                           createBundle("chrome://browser/locale/places/places.properties");
+
         // MOST VISITED
         var smart = {queryId: "MostVisited", // don't change this
                      itemId: null,
-                     title: this._placesBundle.GetStringFromName("mostVisitedTitle"),
+                     title: placesBundle.GetStringFromName("mostVisitedTitle"),
                      uri: this._uri("place:queryType=" +
                                     Ci.nsINavHistoryQueryOptions.QUERY_TYPE_HISTORY +
                                     "&sort=" +
@@ -717,7 +828,7 @@ BrowserGlue.prototype = {
         // RECENTLY BOOKMARKED
         smart = {queryId: "RecentlyBookmarked", // don't change this
                  itemId: null,
-                 title: this._placesBundle.GetStringFromName("recentlyBookmarkedTitle"),
+                 title: placesBundle.GetStringFromName("recentlyBookmarkedTitle"),
                  uri: this._uri("place:folder=BOOKMARKS_MENU" +
                                 "&folder=UNFILED_BOOKMARKS" +
                                 "&folder=TOOLBAR" +
@@ -735,7 +846,7 @@ BrowserGlue.prototype = {
         // RECENT TAGS
         smart = {queryId: "RecentTags", // don't change this
                  itemId: null,
-                 title: this._placesBundle.GetStringFromName("recentTagsTitle"),
+                 title: placesBundle.GetStringFromName("recentTagsTitle"),
                  uri: this._uri("place:"+
                     "type=" +
                     Ci.nsINavHistoryQueryOptions.RESULTS_AS_TAG_QUERY +
@@ -792,20 +903,20 @@ BrowserGlue.prototype = {
       Components.utils.reportError(ex);
     }
     finally {
-      prefBranch.setIntPref(SMART_BOOKMARKS_PREF, SMART_BOOKMARKS_VERSION);
-      prefBranch.QueryInterface(Ci.nsIPrefService).savePrefFile(null);
+      this._prefs.setIntPref(SMART_BOOKMARKS_PREF, SMART_BOOKMARKS_VERSION);
+      this._prefs.QueryInterface(Ci.nsIPrefService).savePrefFile(null);
     }
   },
 
-//@line 848 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
+//@line 957 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
 
   // this returns the most recent non-popup browser window
-  _getMostRecentBrowserWindow : function ()
+  getMostRecentBrowserWindow : function ()
   {
     var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
              getService(Components.interfaces.nsIWindowMediator);
 
-//@line 856 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
+//@line 965 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
     var win = wm.getMostRecentWindow("navigator:browser", true);
 
     // if we're lucky, this isn't a popup, and we can just return this
@@ -819,11 +930,12 @@ BrowserGlue.prototype = {
           win = nextWin;
       }
     }
-//@line 882 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
+//@line 991 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/browser/components/nsBrowserGlue.js"
 
     return win;
   },
 
+
   // for XPCOM
   classDescription: "Firefox Browser Glue Service",
   classID:          Components.ID("{eab9012e-5f74-4cbc-b2b5-a590235513cc}"),
@@ -843,8 +955,126 @@ BrowserGlue.prototype = {
   ]
 }
 
+function GeolocationPrompt() {}
+
+GeolocationPrompt.prototype = {
+  classDescription: "Geolocation Prompting Component",
+  classID:          Components.ID("{C6E8C44D-9F39-4AF7-BCC0-76E38A8310F5}"),
+  contractID:       "@mozilla.org/geolocation/prompt;1",
+
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsIGeolocationPrompt]),
+ 
+  prompt: function(request) {
+    var pm = Cc["@mozilla.org/permissionmanager;1"].getService(Ci.nsIPermissionManager);
+
+    var result = pm.testExactPermission(request.requestingURI, "geo");
+
+    if (result == Ci.nsIPermissionManager.ALLOW_ACTION) {
+      request.allow();
+      return;
+    }
+    
+    if (result == Ci.nsIPermissionManager.DENY_ACTION) {
+      request.cancel();
+      return;
+    }
+
+    function setPagePermission(uri, allow) {
+      if (allow == true)
+        pm.add(uri, "geo", Ci.nsIPermissionManager.ALLOW_ACTION);
+      else
+        pm.add(uri, "geo", Ci.nsIPermissionManager.DENY_ACTION);
+    }
+
+    function getChromeWindow(aWindow) {
+      var chromeWin = aWindow 
+        .QueryInterface(Ci.nsIInterfaceRequestor)
+        .getInterface(Ci.nsIWebNavigation)
+        .QueryInterface(Ci.nsIDocShellTreeItem)
+        .rootTreeItem
+        .QueryInterface(Ci.nsIInterfaceRequestor)
+        .getInterface(Ci.nsIDOMWindow)
+        .QueryInterface(Ci.nsIDOMChromeWindow);
+      return chromeWin;
+    }
+
+    var requestingWindow = request.requestingWindow.top;
+    var chromeWindowObject = getChromeWindow(requestingWindow).wrappedJSObject;
+    var tabbrowser = chromeWindowObject.gBrowser;
+    var browser = tabbrowser.getBrowserForDocument(requestingWindow.document);
+    var notificationBox = tabbrowser.getNotificationBox(browser);
+
+    var notification = notificationBox.getNotificationWithValue("geolocation");
+    if (!notification) {
+      var bundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
+      var browserBundle = bundleService.createBundle("chrome://browser/locale/browser.properties");
+
+      var buttons = [{
+              label: browserBundle.GetStringFromName("geolocation.shareLocation"),
+              accessKey: browserBundle.GetStringFromName("geolocation.shareLocation.accesskey"),
+              callback: function(notification) {
+                  var elements = notification.getElementsByClassName("rememberChoice");
+                  if (elements.length && elements[0].checked)
+                      setPagePermission(request.requestingURI, true);
+                  request.allow(); 
+              },
+          },
+          {
+              label: browserBundle.GetStringFromName("geolocation.dontShareLocation"),
+              accessKey: browserBundle.GetStringFromName("geolocation.dontShareLocation.accesskey"),
+              callback: function(notification) {
+                  var elements = notification.getElementsByClassName("rememberChoice");
+                  if (elements.length && elements[0].checked)
+                      setPagePermission(request.requestingURI, false);
+                  request.cancel();
+              },
+          }];
+      
+      var message = browserBundle.formatStringFromName("geolocation.siteWantsToKnow",
+                                                       [request.requestingURI.host], 1);      
+
+      var newBar = notificationBox.appendNotification(message,
+                                                      "geolocation",
+                                                      "chrome://browser/skin/Geo.png",
+                                                      notificationBox.PRIORITY_INFO_HIGH,
+                                                      buttons);
+
+      // For whatever reason, if we do this immediately
+      // (eg, without the setTimeout), the "link"
+      // element does not show up in the notification
+      // bar.
+      function geolocation_hacks_to_notification () {
+
+        // Never show a remember checkbox inside the private browsing mode
+        var inPrivateBrowsing = Cc["@mozilla.org/privatebrowsing;1"].
+                                getService(Ci.nsIPrivateBrowsingService).
+                                privateBrowsingEnabled;
+        if (!inPrivateBrowsing) {
+          var checkbox = newBar.ownerDocument.createElementNS(XULNS, "checkbox");
+          checkbox.className = "rememberChoice";
+          checkbox.setAttribute("label", browserBundle.GetStringFromName("geolocation.remember"));
+          newBar.appendChild(checkbox);
+        }
+
+        var link = newBar.ownerDocument.createElementNS(XULNS, "label");
+        link.className = "text-link";
+        link.setAttribute("value", browserBundle.GetStringFromName("geolocation.learnMore"));
+
+        var formatter = Cc["@mozilla.org/toolkit/URLFormatterService;1"].getService(Ci.nsIURLFormatter);
+        link.href = formatter.formatURLPref("browser.geolocation.warning.infoURL");
+
+        var description = newBar.ownerDocument.getAnonymousElementByAttribute(newBar, "anonid", "messageText");
+        description.appendChild(link);
+      };
+
+      chromeWindowObject.setTimeout(geolocation_hacks_to_notification, 0);
+
+    }
+  },
+};
+
+
 //module initialization
 function NSGetModule(aCompMgr, aFileSpec) {
-  return XPCOMUtils.generateModule([BrowserGlue]);
+  return XPCOMUtils.generateModule([BrowserGlue, GeolocationPrompt]);
 }
-
