@@ -1,4 +1,4 @@
-//@line 44 "/export/home/mozilla/uild/firefox-3.0.5/src/mozilla/toolkit/mozapps/downloads/content/downloads.js"
+//@line 45 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/toolkit/mozapps/downloads/content/downloads.js"
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Globals
@@ -73,19 +73,7 @@ let gStr = {
 };
 
 // The statement to query for downloads that are active or match the search
-let gStmt = gDownloadManager.DBConnection.createStatement(
-  "SELECT id, target, name, source, state, startTime, endTime, referrer, " +
-         "currBytes, maxBytes, state IN (?1, ?2, ?3, ?4, ?5) isActive " +
-  "FROM moz_downloads " +
-  "ORDER BY isActive DESC, endTime DESC, startTime DESC");
-
-////////////////////////////////////////////////////////////////////////////////
-//// Utility Functions
-
-function getDownload(aID)
-{
-  return document.getElementById("dl" + aID);
-}
+let gStmt = null;
 
 ////////////////////////////////////////////////////////////////////////////////
 //// Start/Stop Observers
@@ -114,10 +102,8 @@ function downloadCompleted(aDownload)
       while (next && next.inProgress)
         next = next.nextSibling;
 
-      // Move the item and color everything after where it moved from
-      let fixup = dl.nextSibling;
+      // Move the item
       gDownloadsView.insertBefore(dl, next);
-      stripeifyList(fixup);
     } else {
       removeFromView(dl);
     }
@@ -244,7 +230,7 @@ function showDownload(aDownload)
 function onDownloadDblClick(aEvent)
 {
   // Only do the default action for double primary clicks
-  if (aEvent.button == 0)
+  if (aEvent.button == 0 && aEvent.target.selected)
     doDefaultForSelected();
 }
 
@@ -259,6 +245,8 @@ function openDownload(aDownload)
       dontAsk = !pref.getBoolPref(PREF_BDM_ALERTONEXEOPEN);
     } catch (e) { }
 
+//@line 301 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/toolkit/mozapps/downloads/content/downloads.js"
+
     if (!dontAsk) {
       var strings = document.getElementById("downloadStrings");
       var name = aDownload.getAttribute("target");
@@ -399,6 +387,7 @@ function Startup()
       gStr[name] = typeof value == "string" ? getStr(value) : value.map(getStr);
   }
 
+  initStatement();
   buildDownloadList(true);
 
   // The DownloadProgressListener (DownloadProgressListener.js) handles progress
@@ -419,19 +408,16 @@ function Startup()
   let obs = Cc["@mozilla.org/observer-service;1"].
             getService(Ci.nsIObserverService);
   obs.addObserver(gDownloadObserver, "download-manager-remove-download", false);
+  obs.addObserver(gDownloadObserver, "private-browsing", false);
 
   // Clear the search box and move focus to the list on escape from the box
   gSearchBox.addEventListener("keypress", function(e) {
     if (e.keyCode == e.DOM_VK_ESCAPE) {
-      // Clear the input as if the user did it
-      gSearchBox.value = "";
-      gSearchBox.doCommand();
-
       // Move focus to the list instead of closing the window
       gDownloadsView.focus();
       e.preventDefault();
     }
-  }, true);
+  }, false);
 }
 
 function Shutdown()
@@ -440,6 +426,7 @@ function Shutdown()
 
   let obs = Cc["@mozilla.org/observer-service;1"].
             getService(Ci.nsIObserverService);
+  obs.removeObserver(gDownloadObserver, "private-browsing");
   obs.removeObserver(gDownloadObserver, "download-manager-remove-download");
 
   clearTimeout(gBuilder);
@@ -451,7 +438,7 @@ let gDownloadObserver = {
   observe: function gdo_observe(aSubject, aTopic, aData) {
     switch (aTopic) {
       case "download-manager-remove-download":
-        // A null subject here indicates "remove all"
+        // A null subject here indicates "remove multiple", so we just rebuild.
         if (!aSubject) {
           // Rebuild the default view
           buildDownloadList(true);
@@ -463,6 +450,27 @@ let gDownloadObserver = {
         let dl = getDownload(id.data);
         removeFromView(dl);
         break;
+      case "private-browsing":
+        if (aData == "enter" || aData == "exit") {
+          // We need to reset the title here, because otherwise the title of
+          // the download manager would still reflect the progress of current
+          // active downloads, if any, after switching the private browsing
+          // mode, even though the downloads will no longer be accessible.
+          // If any download is auto-started after switching the private
+          // browsing mode, the title will be updated as needed by the progress
+          // listener.
+          document.title = document.documentElement.getAttribute("statictitle");
+
+          // We might get this notification before the download manager
+          // service, so the new database connection might not be ready
+          // yet.  Defer this until all private-browsing notifications
+          // have been processed.
+          setTimeout(function() {
+            initStatement();
+            buildDownloadList(true);
+          }, 0);
+        }
+        break;
     }
   }
 };
@@ -1040,9 +1048,6 @@ function removeFromView(aDownload)
   gDownloadsView.removeChild(aDownload);
   gDownloadsView.selectedIndex = Math.min(index, gDownloadsView.itemCount - 1);
 
-  // Color everything after from the newly selected item
-  stripeifyList(gDownloadsView.selectedItem);
-
   // We might have removed the last item, so update the clear list button
   updateClearListButton();
 }
@@ -1158,9 +1163,8 @@ function stepListBuilder(aNumItems) {
     // Make the item and add it to the end if it's active or matches the search
     let item = createDownloadItem(attrs);
     if (item && (isActive || downloadMatchesSearch(item))) {
-      // Add item to the end and color just that one item
+      // Add item to the end
       gDownloadsView.appendChild(item);
-      stripeifyList(item);
     
       // Because of the joys of XBL, we can't update the buttons until the
       // download object is in the document.
@@ -1211,9 +1215,8 @@ function prependList(aDownload)
   // Make the item and add it to the beginning
   let item = createDownloadItem(attrs);
   if (item) {
-    // Add item to the beginning and color the whole list
+    // Add item to the beginning
     gDownloadsView.insertBefore(item, gDownloadsView.firstChild);
-    stripeifyList(item);
     
     // Because of the joys of XBL, we can't update the buttons until the
     // download object is in the document.
@@ -1243,33 +1246,12 @@ function downloadMatchesSearch(aItem)
 
   // Make sure each of the terms are found
   for each (let term in gSearchTerms)
-    if (combinedSearch.search(term) == -1)
+    if (combinedSearch.indexOf(term) == -1)
       return false;
 
   return true;
 }
 
-/**
- * Stripeify the download list by setting or clearing the "alternate" attribute
- * on items starting from a particular item and continuing to the end.
- *
- * @param aItem
- *        Download rishlist item to start stripeifying
- */
-function stripeifyList(aItem)
-{
-  let alt = "alternate";
-  // Set the item to be opposite of the other
-  let flipFrom = function(aOther) aOther && aOther.hasAttribute(alt) ?
-    aItem.removeAttribute(alt) : aItem.setAttribute(alt, "true");
-
-  // Keep coloring items as the opposite of its previous until no more
-  while (aItem) {
-    flipFrom(aItem.previousSibling);
-    aItem = aItem.nextSibling;
-  }
-}
-
 // we should be using real URLs all the time, but until
 // bug 239948 is fully fixed, this will do...
 //
@@ -1305,3 +1287,27 @@ function updateClearListButton()
   // The button is enabled if we have items in the list and we can clean up
   button.disabled = !(gDownloadsView.itemCount && gDownloadManager.canCleanUp);
 }
+
+function getDownload(aID)
+{
+  return document.getElementById("dl" + aID);
+}
+
+/**
+ * Initialize the statement which is used to retrieve the list of downloads.
+ *
+ * This function gets called both at startup, and when entering the private
+ * browsing mode (because the database connection is changed when entering
+ * the private browsing mode, and a new statement should be initialized.
+ */
+function initStatement()
+{
+  if (gStmt)
+    gStmt.finalize();
+
+  gStmt = gDownloadManager.DBConnection.createStatement(
+    "SELECT id, target, name, source, state, startTime, endTime, referrer, " +
+           "currBytes, maxBytes, state IN (?1, ?2, ?3, ?4, ?5) isActive " +
+    "FROM moz_downloads " +
+    "ORDER BY isActive DESC, endTime DESC, startTime DESC");
+}
