@@ -1,4 +1,4 @@
-//@line 38 "/export/home/mozilla/uild/firefox-3.0.5/src/mozilla/toolkit/mozapps/extensions/content/update.js"
+//@line 38 "/export/home/mozilla/packages/BUILD/SFWfirefox/mozilla/toolkit/mozapps/extensions/content/update.js"
 
 // This UI is only opened from the Extension Manager when the app is upgraded.
 
@@ -14,6 +14,9 @@ var gUpdateWizard = {
   // add-ons. When checking for compatible versions this contains only
   // incompatible add-ons.
   items: [],
+  // Contains a list of items that were disabled prior to the application
+  // upgrade.
+  inactiveItemIDs: [],
   // The items that we found updates available for
   itemsToUpdate: [],
   shouldSuggestAutoChecking: false,
@@ -27,6 +30,7 @@ var gUpdateWizard = {
                         .getService(nsIExtensionManager);
     // Retrieve all items in order to sync their app compatibility information
     this.items = em.getItemList(nsIUpdateItem.TYPE_ANY, { });
+    this.inactiveItemIDs = window.arguments[0];
     var pref =
         Components.classes["@mozilla.org/preferences-service;1"].
         getService(Components.interfaces.nsIPrefBranch);
@@ -91,7 +95,7 @@ var gUpdateWizard = {
   errorItems: [],
   showErrors: function (aState, aErrors)
   {
-    openDialog("chrome://mozapps/content/update/errors.xul", "",
+    openDialog("chrome://mozapps/content/extensions/errors.xul", "",
                "modal", { state: aState, errors: aErrors });
   },
 
@@ -167,6 +171,10 @@ var gVersionInfoPage = {
     gUpdateWizard.items = em.getIncompatibleItemList(null, null, null,
                                                      nsIUpdateItem.TYPE_ANY,
                                                      true, { });
+    gUpdateWizard.items = gUpdateWizard.items.filter(function(item) {
+      return gUpdateWizard.inactiveItemIDs.indexOf(item.id) < 0;
+    });
+
     if (gUpdateWizard.items.length > 0) {
       // There are still incompatible addons, inform the user.
       document.documentElement.currentPage =
@@ -355,7 +363,7 @@ var gFoundPage = {
 
     var oneChecked = false;
     var foundUpdates = document.getElementById("found.updates");
-    var updates = foundUpdates.getElementsByTagName("listitem");;
+    var updates = foundUpdates.getElementsByTagName("listitem");
     for (var i = 0; i < updates.length; ++i) {
       if (!updates[i].checked)
         continue;
@@ -390,7 +398,7 @@ var gInstallingPage = {
     this._errors = [];
 
     var foundUpdates = document.getElementById("found.updates");
-    var updates = foundUpdates.getElementsByTagName("listitem");;
+    var updates = foundUpdates.getElementsByTagName("listitem");
     for (var i = 0; i < updates.length; ++i) {
       if (!updates[i].checked)
         continue;
